@@ -1,3 +1,4 @@
+# coding: utf-8
 # highline.rb
 #
 #  Created by James Edward Gray II on 2005-04-26.
@@ -16,6 +17,7 @@ require "highline/question"
 require "highline/menu"
 require "highline/color_scheme"
 require "highline/style"
+require "highline/version"
 
 #
 # A HighLine object is a "high-level line oriented" shell over an input and an
@@ -27,9 +29,6 @@ require "highline/style"
 # checking, convert types, etc.
 #
 class HighLine
-  # The version of the installed library.
-  VERSION = "1.6.20".freeze
-
   # An internal HighLine error.  User code does not need to trap this.
   class QuestionError < StandardError
     # do nothing, just creating a unique error type
@@ -119,17 +118,22 @@ class HighLine
   WHITE_STYLE      = Style.new(:name=>:white,      :builtin=>true, :code=>"\e[37m", :rgb=>[192,192,192])
   # Alias for WHITE, since WHITE is actually a light gray on Macs
   GRAY_STYLE       = Style.new(:name=>:gray,       :builtin=>true, :code=>"\e[37m", :rgb=>[192,192,192])
+  GREY_STYLE       = Style.new(:name=>:grey,       :builtin=>true, :code=>"\e[37m", :rgb=>[192,192,192])
   # On Mac OSX Terminal, this is black foreground, or bright white background.
   # Also used as base for RGB colors, if available
   NONE_STYLE       = Style.new(:name=>:none,       :builtin=>true, :code=>"\e[38m", :rgb=>[  0,  0,  0])
 
-  BASIC_COLORS = %w{BLACK RED GREEN YELLOW BLUE MAGENTA CYAN WHITE GRAY NONE}
+  BASIC_COLORS = %w{BLACK RED GREEN YELLOW BLUE MAGENTA CYAN WHITE GRAY GREY NONE}
 
   colors = BASIC_COLORS.dup
   BASIC_COLORS.each do |color|
     bright_color = "BRIGHT_#{color}"
     colors << bright_color
     const_set bright_color+'_STYLE', const_get(color + '_STYLE').bright
+
+    light_color = "LIGHT_#{color}"
+    colors << light_color
+    const_set light_color+'_STYLE', const_get(color + '_STYLE').light
   end
   COLORS = colors
 
@@ -244,7 +248,7 @@ class HighLine
   #
   # Raises EOFError if input is exhausted.
   #
-  def ask( question, answer_type = String, &details ) # :yields: question
+  def ask( question, answer_type = nil, &details ) # :yields: question
     @question ||= Question.new(question, answer_type, &details)
 
     return gather if @question.gather
@@ -254,7 +258,7 @@ class HighLine
     # the prompt will not be issued. And we have to account for that now.
     # Also, JRuby-1.7's ConsoleReader.readLine() needs to be passed the prompt
     # to handle line editing properly.
-    say(@question) unless ((JRUBY or @question.readline) and @question.echo == true)
+    say(@question) unless ((JRUBY or @question.readline) and (@question.echo == true and @question.limit.nil?))
 
     begin
       @answer = @question.answer_or_default(get_response)
@@ -617,13 +621,15 @@ class HighLine
     statement = format_statement(statement)
     return unless statement.length > 0
 
-      # Don't add a newline if statement ends with whitespace, OR
+    out = (indentation+statement).encode(Encoding.default_external, { :undef => :replace  } )
+
+    # Don't add a newline if statement ends with whitespace, OR
     # if statement ends with whitespace before a color escape code.
     if /[ \t](\e\[\d+(;\d+)*m)?\Z/ =~ statement
-      @output.print(indentation+statement)
+      @output.print(out)
       @output.flush
     else
-      @output.puts(indentation+statement)
+      @output.puts(out)
     end
   end
 
@@ -708,19 +714,19 @@ class HighLine
   private
 
   def format_statement statement
-    statement = statement.dup.to_str
+    statement = String(statement || "").dup
     return statement unless statement.length > 0
 
-  # Allow non-ascii menu prompts in ruby > 1.9.2. ERB eval the menu statement
-    # with the environment's default encoding(usually utf8)
-    statement.force_encoding(Encoding.default_external) if defined?(Encoding) && Encoding.default_external
-
     template  = ERB.new(statement, nil, "%")
     statement = template.result(binding)
 
     statement = wrap(statement) unless @wrap_at.nil?
     statement = page_print(statement) unless @page_at.nil?
 
+    # 'statement' is encoded in US-ASCII when using ruby 1.9.3(-p551)
+    # 'indentation' is correctly encoded (same as default_external encoding)
+    statement = statement.force_encoding(Encoding.default_external)
+
     statement = statement.gsub(/\n(?!$)/,"\n#{indentation}") if @multi_indent
 
     statement
@@ -886,18 +892,20 @@ class HighLine
       else
         raw_no_echo_mode
 
-        line            = ""
+        line            = "".encode(Encoding::BINARY)
         backspace_limit = 0
         begin
 
           while character = get_character(@input)
             # honor backspace and delete
             if character == 127 or character == 8
+              line = line.force_encoding(Encoding.default_external)
               line.slice!(-1, 1)
               backspace_limit -= 1
+              line = line.force_encoding(Encoding::BINARY)
             else
               line << character.chr
-              backspace_limit = line.size
+              backspace_limit = line.dup.force_encoding(Encoding.default_external).size
             end
             # looking for carriage return (decimal 13) or
             # newline (decimal 10) in raw input
@@ -912,10 +920,16 @@ class HighLine
                     # do nothing
                 end
               else
-                if @question.echo == true
-                  @output.print(character.chr)
-                else
-                  @output.print(@question.echo)
+                line_with_next_char_encoded = line.dup.force_encoding(Encoding.default_external)
+                # For multi-byte character, does this
+                #   last character completes the character?
+                # Then print it.
+                if line_with_next_char_encoded.valid_encoding?
+                  if @question.echo == true
+                    @output.print(line_with_next_char_encoded[-1])
+                  else
+                    @output.print(@question.echo)
+                  end
                 end
               end
               @output.flush
@@ -932,7 +946,7 @@ class HighLine
           say("\n")
         end
 
-        @question.change_case(@question.remove_whitespace(line))
+        @question.change_case(@question.remove_whitespace(line.force_encoding(Encoding.default_external)))
       end
     else
       if JRUBY #prompt has not been shown
