@@ -225,22 +225,28 @@ class HighLine
     #
     # Called late in the initialization process to build intelligent
     # responses based on the details of this Question object.
+    # Also used by Menu#update_responses.
     #
-    def build_responses(  )
-      ### WARNING:  This code is quasi-duplicated in     ###
-      ### Menu.update_responses().  Check there too when ###
-      ### making changes!                                ###
-      append_default unless default.nil?
-      @responses = { :ambiguous_completion =>
-                       "Ambiguous choice.  " +
-                       "Please choose one of #{@answer_type.inspect}.",
+    def build_responses(message_source = answer_type, new_hash_wins = false)
+      append_default if [::String, Symbol].include? default.class
+
+      choice_error_str_func = lambda do
+        message_source.is_a?(Array) \
+            ? '[' +  message_source.map { |s| "#{s}" }.join(', ') + ']' \
+            : message_source.inspect
+      end
+
+      old_hash = @responses
+
+      new_hash = { :ambiguous_completion =>
+                       "Ambiguous choice.  Please choose one of " +
+                       choice_error_str_func.call + '.',
                      :ask_on_error         =>
                        "?  ",
                      :invalid_type         =>
-                       "You must enter a valid #{@answer_type}.",
+                       "You must enter a valid #{message_source}.",
                      :no_completion        =>
-                       "You must choose one of " +
-                       "#{@answer_type.inspect}.",
+                       "You must choose one of " + choice_error_str_func.call + '.',
                      :not_in_range         =>
                        "Your answer isn't within the expected range " +
                        "(#{expected_range}).",
@@ -248,10 +254,9 @@ class HighLine
                        "Your entries didn't match.",
                      :not_valid            =>
                        "Your answer isn't valid (must match " +
-                       "#{@validate.inspect})." }.merge(@responses)
-      ### WARNING:  This code is quasi-duplicated in     ###
-      ### Menu.update_responses().  Check there too when ###
-      ### making changes!                                ###
+                       "#{@validate.inspect})." }
+
+      @responses = new_hash_wins ? old_hash.merge(new_hash) : new_hash.merge(old_hash)
     end
 
     #
@@ -436,7 +441,7 @@ class HighLine
     end
 
     # Stringifies the question to be asked.
-    def to_str(  )
+    def to_s
       @question
     end
 
