@@ -80,7 +80,13 @@ class HighLine
         require "dl/import"
 
         module WinAPI
-          extend DL::Importer rescue DL::Importable
+          if defined?(DL::Importer)
+            # Ruby 1.9
+            extend DL::Importer
+          else
+            # Ruby 1.8
+            extend DL::Importable
+          end
           begin
             dlload "msvcrt", "kernel32"
           rescue DL::DLError
@@ -89,6 +95,16 @@ class HighLine
           extern "unsigned long _getch()"
           extern "unsigned long GetConsoleScreenBufferInfo(unsigned long, void*)"
           extern "unsigned long GetStdHandle(unsigned long)"
+
+          # Ruby 1.8 DL::Importable.import does mname[0,1].downcase so FooBar becomes fooBar
+          if defined?(getConsoleScreenBufferInfo)
+            alias_method :GetConsoleScreenBufferInfo, :getConsoleScreenBufferInfo
+            module_function :GetConsoleScreenBufferInfo
+          end
+          if defined?(getStdHandle)
+            alias_method :GetStdHandle, :getStdHandle
+            module_function :GetStdHandle
+          end
         end
       end
 
@@ -207,11 +223,20 @@ class HighLine
         # A Unix savvy method using stty to fetch the console columns, and rows.
         # ... stty does not work in JRuby
         def terminal_size
+          begin
+            require "io/console"
+            winsize = IO.console.winsize.reverse rescue nil
+            return winsize if winsize
+          rescue LoadError
+          end
+
           if /solaris/ =~ RUBY_PLATFORM and
             `stty` =~ /\brows = (\d+).*\bcolumns = (\d+)/
             [$2, $1].map { |c| x.to_i }
+          elsif `stty size` =~ /^(\d+)\s(\d+)$/
+            [$2.to_i, $1.to_i]
           else
-            `stty size`.split.map { |x| x.to_i }.reverse
+            [ 80, 24 ]
           end
         end
       end
