@@ -1,3 +1,4 @@
+# encoding: utf-8
 # tc_highline.rb
 #
 #  Created by James Edward Gray II on 2005-04-26.
@@ -9,6 +10,8 @@ require "test/unit"
 
 require "highline"
 require "stringio"
+require "readline"
+require "tempfile"
 
 if HighLine::CHARACTER_MODE == "Win32API"
   class HighLine
@@ -168,7 +171,7 @@ class TestHighLine < Test::Unit::TestCase
     end
     assert_equal(languages.last, answer)
     assert_equal( "What is your favorite programming language?  " +
-                  "You must choose one of [:Perl, :Python, :Ruby].\n" +
+                  "You must choose one of [Perl, Python, Ruby].\n" +
                   "?  ", @output.string )
   end
   
@@ -224,7 +227,95 @@ class TestHighLine < Test::Unit::TestCase
       assert_equal("", answer)
       assert_equal("Please enter your password: \n", @output.string)
   end
-  
+
+  def test_after_some_chars_backspace_does_not_enter_prompt_when_ascii
+      @input << "apple\b\b\b\b\b\b\b\b\b\b"
+      @input.rewind
+      answer = @terminal.ask("Please enter your password: ") do |q|
+        q.echo = "*"
+      end
+      assert_equal("", answer)
+      assert_equal("apple".size, @output.string.count("\b"))
+  end
+
+  def test_after_some_chars_backspace_does_not_enter_prompt_when_utf8
+      @input << "maçã\b\b\b\b\b\b\b\b"
+      @input.rewind
+      answer = @terminal.ask("Please enter your password: ") do |q|
+        q.echo = "*"
+      end
+      assert_equal("", answer)
+      assert_equal("maçã".size, @output.string.count("\b"))
+  end
+
+  def test_readline_mode
+    # Rubinius seems to be ignoring Readline input
+    # and output assignments. This ruins testing.
+    # but it doesn't mean readline is not working
+    # properly on rubinius.
+
+    return if RUBY_ENGINE == "rbx"
+
+    # Creating Tempfiles here because Readline.input
+    #   and Readline.output only accepts a File object
+    #   as argument (not any duck type as StringIO)
+
+    temp_stdin  = Tempfile.new "temp_stdin"
+    temp_stdout = Tempfile.new "temp_stdout"
+
+    Readline.input  = @input  = File.open(temp_stdin.path, 'w+')
+    Readline.output = @output = File.open(temp_stdout.path, 'w+')
+
+    @terminal = HighLine.new(@input, @output)
+
+    @input << "any input\n"
+    @input.rewind
+
+    answer = @terminal.ask("Prompt:  ") do |q|
+      q.readline = true
+    end
+
+    @output.rewind
+    output = @output.read
+
+    assert_equal "any input", answer
+    assert_match "Prompt:  any input\n", output
+
+    @input.close
+    @output.close
+    Readline.input  = STDIN
+    Readline.output = STDOUT
+  end
+
+  def test_readline_mode_with_limit_set
+    temp_stdin  = Tempfile.new "temp_stdin"
+    temp_stdout = Tempfile.new "temp_stdout"
+
+    Readline.input  = @input  = File.open(temp_stdin.path, 'w+')
+    Readline.output = @output = File.open(temp_stdout.path, 'w+')
+
+    @terminal = HighLine.new(@input, @output)
+
+    @input << "any input\n"
+    @input.rewind
+
+    answer = @terminal.ask("Prompt:  ") do |q|
+      q.limit = 50
+      q.readline = true
+    end
+
+    @output.rewind
+    output = @output.read
+
+    assert_equal "any input", answer
+    assert_equal "Prompt:  any input\n", output
+
+    @input.close
+    @output.close
+    Readline.input  = STDIN
+    Readline.output = STDOUT
+  end
+
   def test_readline_on_non_echo_question_has_prompt
     @input << "you can't see me"
     @input.rewind
@@ -327,6 +418,30 @@ class TestHighLine < Test::Unit::TestCase
                   @terminal.uncolor("This should be \e[38;5;137mrgb_906030\e[0m!\n") 
                 )
   end
+
+  def test_grey_is_the_same_of_gray
+    @terminal.say("<%= GRAY %>")
+    gray_code = @output.string.dup
+    @output.truncate(@output.rewind)
+
+    @terminal.say("<%= GREY %>")
+    grey_code = @output.string.dup
+    @output.truncate(@output.rewind)
+
+    assert_equal gray_code, grey_code
+  end
+
+  def test_light_is_the_same_as_bright
+    @terminal.say("<%= BRIGHT_BLUE %>")
+    bright_blue_code = @output.string.dup
+    @output.truncate(@output.rewind)
+
+    @terminal.say("<%= LIGHT_BLUE %>")
+    light_blue_code = @output.string.dup
+    @output.truncate(@output.rewind)
+
+    assert_equal bright_blue_code, light_blue_code
+  end
                                   
   def test_confirm
     @input << "junk.txt\nno\nsave.txt\ny\n"
@@ -380,6 +495,51 @@ class TestHighLine < Test::Unit::TestCase
                   @output.string )
   end
 
+  def test_default_with_String
+    @input << "\n"
+    @input.rewind
+
+    answer = @terminal.ask("Question:  ") do |q|
+      q.default = "string"
+    end
+
+    assert_equal "string", answer
+    assert_equal "Question:  |string|  ", @output.string
+  end
+
+  def test_default_with_Symbol
+    # With a Symbol, it should show up the String version
+    #   at prompt, but return the Symbol as answer
+
+    @input << "\n"
+    @input.rewind
+
+    answer = @terminal.ask("Question:  ") do |q|
+      q.default = :string
+    end
+
+    assert_equal :string, answer
+    assert_equal "Question:  |string|  ", @output.string
+  end
+
+  def test_default_with_non_String_objects
+    # With a non-string object, it should not show
+    #   any 'default' at prompt line. And should
+    #   return the "default" object, without conversion.
+
+    @input << "\n"
+    @input.rewind
+
+    default_non_string_object = Object.new
+
+    answer = @terminal.ask("Question:  ") do |q|
+      q.default = default_non_string_object
+    end
+
+    assert_equal default_non_string_object, answer
+    assert_equal "Question:  ", @output.string
+  end
+
   def test_string_preservation
     @input << "Maybe\nYes\n"
     @input.rewind
@@ -427,7 +587,7 @@ class TestHighLine < Test::Unit::TestCase
       q.glob      = "*.rb"
     end
     assert_instance_of(File, file)
-    assert_equal("# tc_highline.rb\n", file.gets)
+    assert_equal("# encoding: utf-8\n", file.gets)
     file.close
 
     @input.rewind
@@ -755,6 +915,54 @@ class TestHighLine < Test::Unit::TestCase
     assert_equal("Pick a letter or number:  \n", @output.string)
   end
   
+  def test_correct_string_encoding_when_echo_false
+    @input << "ação\r" # An UTF-8 portuguese word for 'action'
+    @input.rewind
+
+    answer = @terminal.ask("Please enter your password:  ") do |q|
+      q.echo = false
+    end
+
+    assert_equal "ação", answer
+    assert_equal Encoding::default_external, answer.encoding
+  end
+
+  def test_backspace_with_ascii_when_echo_false
+    @input << "password\b\r"
+    @input.rewind
+
+    answer = @terminal.ask("Please enter your password:  ") do |q|
+      q.echo = false
+    end
+
+    assert_not_equal("password", answer)
+    assert_equal("passwor", answer)
+  end
+
+  def test_backspace_with_utf8_when_echo_false
+    @input << "maçã\b\r"
+    @input.rewind
+
+    answer = @terminal.ask("Please enter your password:  ") do |q|
+      q.echo = false
+    end
+
+    assert_not_equal("maçã", answer)
+    assert_equal("maç", answer)
+  end
+
+  def test_echoing_with_utf8_when_echo_is_star
+    @input << "maçã\r"
+    @input.rewind
+
+    answer = @terminal.ask("Type:  ") do |q|
+      q.echo = "*"
+    end
+
+    assert_equal("Type:  ****\n", @output.string)
+    assert_equal("maçã", answer)
+  end
+
   def test_paging
     @terminal.page_at = 22
 
@@ -877,7 +1085,7 @@ class TestHighLine < Test::Unit::TestCase
     assert_equal(:generate, answer)
     assert_equal( "Select a mode:  " +
                   "Ambiguous choice.  " +
-                  "Please choose one of [:generate, :gentle].\n" +
+                  "Please choose one of [generate, gentle].\n" +
                   "?  ", @output.string )
   end
   
@@ -933,6 +1141,24 @@ class TestHighLine < Test::Unit::TestCase
     colorized = @terminal.color("This will have a newline.", :green)
     @terminal.say(colorized)
     assert_equal("\e[32mThis will have a newline.\e[0m\n", @output.string)
+
+    @output.truncate(@output.rewind)
+
+    assert_nothing_raised { @terminal.say(nil) }
+    assert_equal("", @output.string)
+  end
+
+  def test_say_handles_non_string_argument
+    integer = 10
+    hash    = { :a => 20 }
+
+    assert_nothing_raised { @terminal.say(integer) }
+    assert_equal String(integer), @output.string.chomp
+
+    @output.truncate(@output.rewind)
+
+    assert_nothing_raised { @terminal.say(hash) }
+    assert_equal String(hash), @output.string.chomp
   end
 
   def test_terminal_size
